<?php

declare(strict_types=1);

namespace Drupal\commerce_paytrail\PluginForm\OffsiteRedirect;

use Drupal\commerce_payment\Entity\PaymentInterface;
use Drupal\commerce_payment\PluginForm\PaymentOffsiteForm;
use Drupal\commerce_paytrail\RequestBuilder\TokenRequestBuilderInterface;
use Drupal\Core\DependencyInjection\AutowireTrait;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Paytrail\SDK\Client;
use Psr\Log\LoggerInterface;
use Symfony\Component\DependencyInjection\Attribute\Autowire;

/**
 * Provides the Paytrail token off-site form.
 */
final class PaytrailTokenForm extends PaymentOffsiteForm implements ContainerInjectionInterface {

  use StringTranslationTrait;
  use AutowireTrait;

  /**
   * Constructs a new instance.
   *
   * @param \Drupal\commerce_paytrail\RequestBuilder\TokenRequestBuilderInterface $tokenRequestBuilder
   *   The token request builder.
   * @param \Psr\Log\LoggerInterface $logger
   *   The logger service.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger service.
   */
  public function __construct(
    private TokenRequestBuilderInterface $tokenRequestBuilder,
    #[Autowire('@logger.channel.commerce_paytrail')] private LoggerInterface $logger,
    private MessengerInterface $messenger,
  ) {
  }

  /**
   * Builds the configuration form.
   *
   * @param array<string, mixed> $form
   *   The form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   *
   * @return array<string, mixed>
   *   The form.
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) : array {
    $form = parent::buildConfigurationForm($form, $form_state);
    $form['payment_details'] = [];

    assert($this->entity instanceof PaymentInterface);

    if (!$order = $this->entity->getOrder()) {
      $this->logger
        ->error(sprintf('Payment %s has no order referenced.', $this->entity->id()));
      $this->messenger->addError(
        $this->t('The provided payment has no order referenced. Please contact store administration if the problem persists.')
      );
      return $form;
    }

    $request = $this->tokenRequestBuilder
      ->createAddCardFormForOrder($order, $form['#capture']);

    $uri = Client::API_ENDPOINT . '/tokenization/addcard-form';
    return $this->buildRedirectForm($form, $form_state, $uri, $request->toArray(), self::REDIRECT_POST);
  }

}
